USE [PCM_DBSQL]
GO

IF (SELECT CONVERT(int,SUBSTRING(@@VERSION,22,5)))=2000
BEGIN
if exists (select * from dbo.sysobjects where id = object_id(N'[PigChamp_Export]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
drop table [PigChamp_Export]
END
ELSE
BEGIN
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PigChamp_Export]') AND type in (N'U'))
DROP TABLE [dbo].[PigChamp_Export]
END
GO

CREATE TABLE [dbo].[PigChamp_Export](
	[RecID] [int] IDENTITY(1,1) NOT NULL,
	[TagNumber] [nvarchar](50) NOT NULL,
	[EventDate] [smalldatetime] NOT NULL,
	[EventOrder] [smallint] NOT NULL,
	[ExportString] [nvarchar](500) NOT NULL,
 CONSTRAINT [PK_PigChamp_Export] PRIMARY KEY CLUSTERED 
(
	 [EventDate] ASC, [EventOrder] ASC, [TagNumber] ASC, [RecID] ASC
) ON [PRIMARY]
) ON [PRIMARY]

GO 

ALTER  PROCEDURE stp_ExportToPigChamp (@DateFrom smalldatetime, @DateTo smalldatetime)
AS
SET NOCOUNT ON

DECLARE @ODR_Arrival smallint, @ODR_Pedigree smallint, @ODR_Service smallint, @ODR_Return smallint, @ODR_Farrow smallint, @ODR_Day21 smallint
DECLARE @ODR_FosterOn smallint, @ODR_FosterOff smallint
DECLARE @ODR_Wean smallint, @ODR_PigletDeath smallint, @ODR_Disease smallint, @ODR_Treatment smallint, @ODR_Leave smallint
SET @ODR_Pedigree = 1
SET @ODR_Arrival = 2
SET @ODR_Return = 3
SET @ODR_Service = 4
SET @ODR_Farrow = 5
SET @ODR_FosterOff = 6
SET @ODR_FosterOn = 7
SET @ODR_Day21 = 8
SET @ODR_Wean = 9
SET @ODR_PigletDeath = 10
SET @ODR_Disease = 11
SET @ODR_Treatment = 12
SET @ODR_Leave = 13

DECLARE @SV_Returned smallint, @SV_Aborted smallint, @SV_Doubtful smallint, @SV_NIP smallint
SET @SV_Returned = 1
SET @SV_Aborted = 2
SET @SV_Doubtful = 4
SET @SV_NIP = 5

DECLARE @FarmID int
SET @FarmID = dbo.stf_GetCurrentFarm()

DELETE FROM dbo.PigChamp_Export


--Arrivals
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		TagNumber, DateIn, @ODR_Arrival, TagNumber + ';' 
					+ CONVERT(nvarchar(8),DateIn,1) + ';' 
					+ 'ENTER' + ';' 
					+ CASE WHEN IsBoar=1 THEN 'MALE' ELSE ' FEMALE' END + ';'
					+ CONVERT(nvarchar(2), ISNULL(ArrivalParity,0)) + ';' 
					+ CONVERT(nvarchar(8), DateIn,1)
FROM		dbo.Animals_ALL
WHERE		Farm_KEY = @FarmID and DateIn between @DateFrom and @DateTo


--Pedigree
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		a.TagNumber, a.DateBirth, @ODR_Pedigree, a.TagNumber + ';' 
					+ 'HEADER' + ';' 
					+ CONVERT(nvarchar(8), a.DateBirth,1) + ';' 
					+ ISNULL(b.LookupNAME, '') + ';'
					+ ISNULL(p.LookupNAME, '') + ';'
					+ a.TattooNumber + ';'
					+ ISNULL(a.Sire, '') + ';'
					+ ISNULL(a.Dam, '')
FROM		dbo.Animals_ALL a					
LEFT JOIN	dbo.LKP_Breeds b ON a.Breed_KEY = b.LookupKEY
LEFT JOIN	dbo.LKP_Places p ON a.Origin_KEY = p.LookupKEY
WHERE		a.DateBirth IS NOT NULL AND a.TattooNumber IS NOT NULL AND Farm_KEY = @FarmID
			and DateIn between @DateFrom and @DateTo


--Services
INSERT INTO 	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, s.ServiceDate, @ODR_Service, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),s.ServiceDate,1) + ';'
						+ CASE WHEN m.AIService=1 THEN 'AI' ELSE 'MATING' END + ';'
						+ br.TagNumber + ';'
						+ ISNULL(o.LookupNAME,'') + ';'
						+ CONVERT(nvarchar(2),ISNULL(m.ServiceTiming_KEY,1) * 4)
FROM		dbo.Matings_ALL m
JOIN		dbo.Services_ALL s ON m.Service_KEY = s.ServiceID
JOIN		dbo.Animals_ALL sw ON s.Sow_KEY = sw.AnimalID
JOIN		dbo.Animals_ALL br ON m.Boar_KEY = br.AnimalID
LEFT JOIN	dbo.LKP_Operators_ALL o ON m.Operator_KEY = o.LookupKEY
WHERE		s.Farm_KEY = @FarmID and s.ServiceDate between @DateFrom and @DateTo


-- Services with no mating record
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, s.ServiceDate, @ODR_Service, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),s.ServiceDate,1) + ';'
						+ 'MATING' + ';'
						+ ';'
						+ ';'
						-- no service time 
FROM		dbo.Services_ALL s 
JOIN		dbo.Animals_ALL sw ON s.Sow_KEY = sw.AnimalID
WHERE		s.Farm_KEY = @FarmID and s.ServiceDate between @DateFrom and @DateTo
			AND NOT EXISTS(SELECT * FROM dbo.Matings_ALL m WHERE m.Service_KEY = s.ServiceID)


--Returns and Abortions
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, s.Result_Date, @ODR_Return, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8), s.Result_Date,1) + ';'
						+ CASE WHEN s.Result_KEY = @SV_Aborted THEN 'ABORT' ELSE 'NOT IN PIG' END 
FROM		dbo.Services_ALL s
JOIN		dbo.Animals_ALL sw ON s.Sow_KEY = sw.AnimalID
WHERE		s.Farm_KEY = @FarmID 
			AND s.Result_Date between @DateFrom and @DateTo
			and	s.Result_KEY IN (@SV_Returned, @SV_Aborted, @SV_Doubtful, @SV_NIP ) 


--Farrowings (convert litterwt to livewt)
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, f.DateBirth, @ODR_Farrow, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),f.DateBirth,1) + ';'
						+ 'FARROW' + ';'
						+ CONVERT(nvarchar(2), CONVERT(smallint,ISNULL(f.Males,0)) + CONVERT(smallint,ISNULL(f.Females,0))) + ';'
						+ CONVERT(nvarchar(2), CONVERT(smallint,ISNULL(f.Dead,0))) + ';'
						+ CONVERT(nvarchar(2), CONVERT(smallint,ISNULL(f.Mummified,0))) + ';'
						+ ISNULL(CONVERT(nvarchar(10), CONVERT(decimal(9,2),  (f.Males + f.Females) * f.LitterWt / NULLIF(f.Males + f.Females + f.Dead,0) )),'') + ';'
						+ ISNULL(f.LitterNumber,'') + ';'
						+ ';' 
						+ ';'
						+ ISNULL(lc.LookupName,'') + ';'
						+ ';'						
FROM		dbo.Litters_ALL f
JOIN		dbo.Animals_ALL sw ON f.Sow_KEY = sw.AnimalID
LEFT JOIN	dbo.LKP_Locations_ALL lc ON f.Location_KEY = lc.LookupKEY
WHERE		f.Farm_KEY = @FarmID and f.DateBirth between @DateFrom and @DateTo


--Foster Off
INSERT		dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, f.FosterDate, @ODR_FosterOFF, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),f.FosterDate,1) + ';'
						+ 'FO' + ';'
						+ '-' + CONVERT(nvarchar(2),f.NumberOfPiglets) + ';'
						+ ''
FROM		dbo.Fosters_ALL f
JOIN		dbo.Litters_ALL L on f.FromLitter_KEY = L.LitterID
JOIN		dbo.Animals_ALL sw on L.Sow_KEY = sw.AnimalID
WHERE		f.Farm_KEY = @FarmID and f.FosterDate between @DateFrom and @DateTo

--Foster On
INSERT		dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, f.FosterDate, @ODR_FosterON, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),f.FosterDate,1) + ';'
						+ 'FO' + ';'
						+ CONVERT(nvarchar(2),f.NumberOfPiglets) + ';'
						+ ''
FROM		dbo.Fosters_ALL f
JOIN		dbo.Litters_ALL L on f.ToLitter_KEY = L.LitterID
JOIN		dbo.Animals_ALL sw on L.Sow_KEY = sw.AnimalID
WHERE		f.Farm_KEY = @FarmID and f.FosterDate between @DateFrom and @DateTo


-- Day 21 Weights
INSERT		dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, w.Day21Date, @ODR_Day21, sw.TagNumber + ';' 
								+ CONVERT(nvarchar(8),w.Day21Date,1) + ';'
								+ f.LitterNumber + ';'
								+ CONVERT(nvarchar(2),w.Day21Number) + ';'
   							+ CONVERT(nvarchar(10), CONVERT(decimal(9,2), w.Day21Weight)) 
FROM		dbo.GEP_Day21_ALL w
JOIN		dbo.Litters_ALL f ON w.Litter_KEY = f.LitterID
JOIN		dbo.Animals_ALL sw ON f.Sow_KEY = sw.AnimalID
WHERE		f.Farm_KEY = @FarmID and w.Day21Date between @DateFrom and @DateTo


-- Weaning 
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, w.WeaningDate, @ODR_Wean, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),w.WeaningDate,1) + ';'
						+ CASE WHEN w.ExtraWeaning = 1 THEN 'PART WEAN' ELSE 'WEAN' END + ';'
						+ CONVERT(nvarchar(2),w.PigletsWeaned) + ';'
						+ ISNULL(CONVERT(nvarchar(10), CONVERT(decimal(9,2), w.WeightWeaned)), '')	
FROM		dbo.Weanings_ALL w
JOIN		dbo.Litters_ALL f ON w.Litter_KEY = f.LitterID
JOIN		dbo.Animals_ALL sw ON f.Sow_KEY = sw.AnimalID
WHERE		f.Farm_KEY = @FarmID and w.WeaningDate between @DateFrom and @DateTo


--Wean animals that left herd still suckling
DECLARE 	@LeftHerd TABLE (Sow_KEY int, DateOut smalldatetime, LastDate smalldatetime, Litter_KEY int)
INSERT  	@LeftHerd(Sow_KEY, DateOut, LastDate)
SELECT		sw.AnimalID, sw.DateOut, MAX(f.DateBirth)
FROM		dbo.Animals_ALL sw
JOIN		dbo.Litters_ALL f ON sw.AnimalID = f.Sow_KEY
WHERE		sw.DateOut between @DateFrom and @DateTo 
			AND sw.Farm_KEY = @FarmID
GROUP BY	sw.AnimalID, sw.DateOut

UPDATE		sw
SET			sw.Litter_KEY = f.LitterID
FROM		@LeftHerd sw
JOIN		dbo.Litters_ALL f ON sw.Sow_KEY = f.Sow_KEY AND sw.LastDate = f.DateBirth

DELETE 		sw
FROM		@LeftHerd sw
JOIN		dbo.Weanings_ALL w ON sw.Litter_KEY = w.Litter_KEY
WHERE		w.ExtraWeaning = 0

INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, sw.DateOut, @ODR_Wean, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8),sw.DateOut,1) + ';'
						+ 'WEAN' + ';'
						+ '0;'
						-- + dont know weaning weight
FROM		@LeftHerd s
JOIN		dbo.Animals_ALL sw ON s.Sow_KEY = sw.AnimalID


-- Piglet Deaths
INSERT		dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		sw.TagNumber, pd.DateDied, @ODR_PigletDeath, sw.TagNumber + ';'
						+ CONVERT(nvarchar(8), pd.DateDied,1) + ';'
						+ 'PIG DEATH' + ';'
						+ CONVERT(nvarchar(2), pd.NumberDied) + ';' 
						+ c.LookupNAME					
FROM		dbo.Piglet_Deaths_ALL pd
JOIN		dbo.Animals_ALL sw ON pd.Sow_KEY = sw.AnimalID
JOIN		dbo.LKP_Conditions c ON pd.Condition_KEY = c.LookupKEY
WHERE		sw.Farm_KEY = @FarmID and pd.DateDied between @DateFrom and @DateTo


--Treatments
-- Condition Treated
INSERT  	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		a.TagNumber, t.TreatmentDate, @ODR_Disease, a.TagNumber + ';' 
							+ CONVERT(nvarchar(8), t.TreatmentDate, 1) + ';'								
							+ 'DISEASE' + ';'
							+ c.LookupNAME
FROM		dbo.Treatments_Animals_ALL t
JOIN		dbo.Animals_ALL a ON t.Animal_KEY = a.AnimalID
JOIN		dbo.LKP_Conditions c ON t.Condition_KEY = c.LookupKEY
JOIN		dbo.LKP_Drugs d ON t.Drug_KEY = d.LookupKEY
WHERE		t.Farm_KEY = @FarmID and t.TreatmentDate between @DateFrom and @DateTo

/*
--Drug Used
INSERT INTO 	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		a.TagNumber, t.TreatmentDate, @ODR_Treatment, a.TagNumber + ';' 
							+ CONVERT(nvarchar(8), t.TreatmentDate, 1) + ';'								
							+ 'TREATMENT' + ';'
							+ d.LookupNAME
FROM		dbo.Treatments_Animals_ALL t
INNER JOIN	dbo.Animals_ALL a ON t.Animal_KEY = a.AnimalID
INNER JOIN	dbo.LKP_Conditions c ON t.Condition_KEY = c.LookupKEY
INNER JOIN	dbo.LKP_Drugs d ON t.Drug_KEY = d.LookupKEY
WHERE		t.Farm_KEY = @FarmID
*/


-- Departures
INSERT INTO 	dbo.PigChamp_Export(TagNumber, EventDate, EventOrder, ExportString)
SELECT		a.TagNumber, a.DateOut, @ODR_Leave, a.TagNumber + ';' 
					+ CONVERT(nvarchar(8),a.DateOut,1) + ';' 
					+ 'REMOVE' + ';' 
					+ ';'
					+ c.LookupNAME	 + ';'					
FROM		dbo.Animals_ALL a
INNER JOIN	dbo.LKP_Conditions c ON a.ReasonOut_KEY = c.LookupKEY
WHERE		a.Farm_KEY = @FarmID and a.DateOut between @DateFrom and @DateTo



-- What if no sow specified for piglet death?
-- FH Movements?
-- Header record - order of fields?

RETURN
GO


ALTER  VIEW dbo.PigChamp_ExportQRY
AS
SELECT     TOP 100 PERCENT ExportString
FROM         dbo.PigChamp_Export
ORDER BY EventDate, EventOrder, TagNumber


GO
